package com.jiuqi.util;

import java.net.MalformedURLException;
import java.net.URL;
import java.io.File;
import java.nio.channels.WritableByteChannel;
import java.nio.channels.FileChannel;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.io.OutputStream;
import java.nio.channels.Channels;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;

public class FileClassLoader extends ClassLoader
{
    private String baseDir;
    
    public FileClassLoader(final String baseDir) {
        this.baseDir = baseDir;
    }
    
    public FileClassLoader(final ClassLoader parent, final String baseDir) {
        super(parent);
        this.baseDir = baseDir;
    }
    
    @Override
    protected Class findClass(final String name) throws ClassNotFoundException {
        final byte[] bytes = this.loadClassBytes(name);
        Class theClass = null;
        try {
            theClass = this.defineClass(name, bytes, 0, bytes.length);
        }
        catch (final Exception e) {
            e.printStackTrace();
        }
        if (theClass == null) {
            throw new ClassFormatError();
        }
        return theClass;
    }
    
    private byte[] loadClassBytes(final String className) throws ClassNotFoundException {
        try {
            final String classFile = this.getClassFile(className);
            final FileInputStream fis = new FileInputStream(classFile);
            final FileChannel fileC = fis.getChannel();
            final ByteArrayOutputStream baos = new ByteArrayOutputStream();
            final WritableByteChannel outC = Channels.newChannel(baos);
            final ByteBuffer buffer = ByteBuffer.allocateDirect(1024);
            while (true) {
                final int i = fileC.read(buffer);
                if (i == 0 || i == -1) {
                    break;
                }
                buffer.flip();
                outC.write(buffer);
                buffer.clear();
            }
            fis.close();
            return baos.toByteArray();
        }
        catch (final IOException fnfe) {
            throw new ClassNotFoundException(className, fnfe);
        }
    }
    
    private String getClassFile(String name) {
        final StringBuffer sb = new StringBuffer(this.baseDir);
        name = String.valueOf(name.replace('.', File.separatorChar)) + ".class";
        sb.append(String.valueOf(File.separator) + name);
        return sb.toString();
    }
    
    @Override
    protected URL findResource(final String name) {
        try {
            URL url = super.findResource(name);
            if (url != null) {
                return url;
            }
            url = new URL("file:///" + this.converName(name));
            return url;
        }
        catch (final MalformedURLException ex) {
            return null;
        }
    }
    
    private String converName(String name) {
        final StringBuffer sb = new StringBuffer(this.baseDir);
        name = name.replace('.', File.separatorChar);
        sb.append(String.valueOf(File.separator) + name);
        return sb.toString();
    }
}
